@@ -32,6 +32,7 @@ struct instproxy_status_data {
 	instproxy_client_t client;
 	instproxy_status_cb_t cbfunc;
 	char *operation;
+	void *user_data;
 };
 
 /**
@@ -272,7 +273,7 @@ leave_unlock:
  *        in async mode or shown in debug messages in sync mode. 
  * @param user_data Callback data passed to status_cb.
  */
-static instproxy_error_t instproxy_perform_operation(instproxy_client_t client, instproxy_status_cb_t status_cb, const char *operation)
+static instproxy_error_t instproxy_perform_operation(instproxy_client_t client, instproxy_status_cb_t status_cb, const char *operation, void *user_data)
 {
 	instproxy_error_t res = INSTPROXY_E_UNKNOWN_ERROR;
 	int ok = 1;
@@ -289,7 +290,7 @@ static instproxy_error_t instproxy_perform_operation(instproxy_client_t client,
 		if (dict) {
 			/* invoke callback function */
 			if (status_cb) {
-				status_cb(operation, dict);
+				status_cb(operation, dict, user_data);
 			}
 			/* check for 'Error', so we can abort cleanly */
 			plist_t err = plist_dict_get_item(dict, "Error");
@@ -353,7 +354,7 @@ static gpointer instproxy_status_updater(gpointer arg)
 	struct instproxy_status_data *data = (struct instproxy_status_data*)arg;
 
 	/* run until the operation is complete or an error occurs */
-	(void)instproxy_perform_operation(data->client, data->cbfunc, data->operation);
+	(void)instproxy_perform_operation(data->client, data->cbfunc, data->operation, data->user_data);
 
 	/* cleanup */
 	instproxy_lock(data->client);
@@ -384,7 +385,7 @@ static gpointer instproxy_status_updater(gpointer arg)
  *         when the operation completed successfully (sync).
  *         An INSTPROXY_E_* error value is returned if an error occured.
  */
-static instproxy_error_t instproxy_create_status_updater(instproxy_client_t client, instproxy_status_cb_t status_cb, const char *operation)
+static instproxy_error_t instproxy_create_status_updater(instproxy_client_t client, instproxy_status_cb_t status_cb, const char *operation, void *user_data)
 {
 	instproxy_error_t res = INSTPROXY_E_UNKNOWN_ERROR;
 	if (status_cb) {
@@ -394,6 +395,7 @@ static instproxy_error_t instproxy_create_status_updater(instproxy_client_t clie
 			data->client = client;
 			data->cbfunc = status_cb;
 			data->operation = strdup(operation);
+			data->user_data = user_data;
 
 			client->status_updater = g_thread_create(instproxy_status_updater, data, TRUE, NULL);
 			if (client->status_updater) {
@@ -402,7 +404,7 @@ static instproxy_error_t instproxy_create_status_updater(instproxy_client_t clie
 		}
 	} else {
 		/* sync mode */
-		res = instproxy_perform_operation(client, NULL, operation);
+		res = instproxy_perform_operation(client, NULL, operation, NULL);
 	}
 	return res;
 }
@@ -422,7 +424,7 @@ static instproxy_error_t instproxy_create_status_updater(instproxy_client_t clie
  * @return INSTPROXY_E_SUCCESS on success or an INSTPROXY_E_* error value if
  *     an error occured. 
  */
-static instproxy_error_t instproxy_install_or_upgrade(instproxy_client_t client, const char *pkg_path, plist_t client_options, instproxy_status_cb_t status_cb, const char *command)
+static instproxy_error_t instproxy_install_or_upgrade(instproxy_client_t client, const char *pkg_path, plist_t client_options, instproxy_status_cb_t status_cb, const char *command, void *user_data)
 {
 	if (!client || !client->parent || !pkg_path) {
 		return INSTPROXY_E_INVALID_ARG;
@@ -440,7 +442,7 @@ static instproxy_error_t instproxy_install_or_upgrade(instproxy_client_t client,
 		return res;
 	}
 
-	return instproxy_create_status_updater(client, status_cb, command);
+	return instproxy_create_status_updater(client, status_cb, command, user_data);
 }
 
 /**
@@ -467,9 +469,9 @@ static instproxy_error_t instproxy_install_or_upgrade(instproxy_client_t client,
  *     created successfully; any error occuring during the operation has to be
  *     handled inside the specified callback function.
  */
-instproxy_error_t instproxy_install(instproxy_client_t client, const char *pkg_path, plist_t client_options, instproxy_status_cb_t status_cb)
+instproxy_error_t instproxy_install(instproxy_client_t client, const char *pkg_path, plist_t client_options, instproxy_status_cb_t status_cb, void *user_data)
 {
-	return instproxy_install_or_upgrade(client, pkg_path, client_options, status_cb, "Install");
+	return instproxy_install_or_upgrade(client, pkg_path, client_options, status_cb, "Install", user_data);
 }
 
 /**
@@ -498,9 +500,9 @@ instproxy_error_t instproxy_install(instproxy_client_t client, const char *pkg_p
  *     created successfully; any error occuring during the operation has to be
  *     handled inside the specified callback function.
  */
-instproxy_error_t instproxy_upgrade(instproxy_client_t client, const char *pkg_path, plist_t client_options, instproxy_status_cb_t status_cb)
+instproxy_error_t instproxy_upgrade(instproxy_client_t client, const char *pkg_path, plist_t client_options, instproxy_status_cb_t status_cb, void *user_data)
 {
-	return instproxy_install_or_upgrade(client, pkg_path, client_options, status_cb, "Upgrade");
+	return instproxy_install_or_upgrade(client, pkg_path, client_options, status_cb, "Upgrade", user_data);
 }
 
 /**
@@ -522,7 +524,7 @@ instproxy_error_t instproxy_upgrade(instproxy_client_t client, const char *pkg_p
  *     created successfully; any error occuring during the operation has to be
  *     handled inside the specified callback function.
  */
-instproxy_error_t instproxy_uninstall(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb)
+instproxy_error_t instproxy_uninstall(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb, void *user_data)
 {
 	if (!client || !client->parent || !appid) {
 		return INSTPROXY_E_INVALID_ARG;
@@ -548,7 +550,7 @@ instproxy_error_t instproxy_uninstall(instproxy_client_t client, const char *app
 		return res;
 	}
 
-	return instproxy_create_status_updater(client, status_cb, "Uninstall");
+	return instproxy_create_status_updater(client, status_cb, "Uninstall", user_data);
 }
 
 /**
@@ -615,7 +617,7 @@ leave_unlock:
  *     created successfully; any error occuring during the operation has to be
  *     handled inside the specified callback function.
  */
-instproxy_error_t instproxy_archive(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb)
+instproxy_error_t instproxy_archive(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb, void *user_data)
 {
 	if (!client || !client->parent || !appid)
 		return INSTPROXY_E_INVALID_ARG;
@@ -632,7 +634,7 @@ instproxy_error_t instproxy_archive(instproxy_client_t client, const char *appid
 		debug_info("could not send plist, error %d", res);
 		return res;
 	}
-	return instproxy_create_status_updater(client, status_cb, "Archive");
+	return instproxy_create_status_updater(client, status_cb, "Archive", user_data);
 }
 
 /**
@@ -656,7 +658,7 @@ instproxy_error_t instproxy_archive(instproxy_client_t client, const char *appid
  *     created successfully; any error occuring during the operation has to be
  *     handled inside the specified callback function.
  */
-instproxy_error_t instproxy_restore(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb)
+instproxy_error_t instproxy_restore(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb, void *user_data)
 {
 	if (!client || !client->parent || !appid)
 		return INSTPROXY_E_INVALID_ARG;
@@ -673,7 +675,7 @@ instproxy_error_t instproxy_restore(instproxy_client_t client, const char *appid
 		debug_info("could not send plist, error %d", res);
 		return res;
 	}
-	return instproxy_create_status_updater(client, status_cb, "Restore");
+	return instproxy_create_status_updater(client, status_cb, "Restore", user_data);
 }
 
 /**
@@ -697,7 +699,7 @@ instproxy_error_t instproxy_restore(instproxy_client_t client, const char *appid
  *     created successfully; any error occuring during the operation has to be
  *     handled inside the specified callback function.
  */
-instproxy_error_t instproxy_remove_archive(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb)
+instproxy_error_t instproxy_remove_archive(instproxy_client_t client, const char *appid, plist_t client_options, instproxy_status_cb_t status_cb, void *user_data)
 {
 	if (!client || !client->parent || !appid)
 		return INSTPROXY_E_INVALID_ARG;
@@ -714,7 +716,7 @@ instproxy_error_t instproxy_remove_archive(instproxy_client_t client, const char
 		debug_info("could not send plist, error %d", res);
 		return res;
 	}
-	return instproxy_create_status_updater(client, status_cb, "RemoveArchive");
+	return instproxy_create_status_updater(client, status_cb, "RemoveArchive", user_data);
 }
 
 /**
